/*===========================================================================

FILE: NetworkSample.c
===========================================================================*/


/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEModGen.h"          // Module interface definitions
#include "AEEAppGen.h"          // Applet interface definitions
#include "AEEShell.h"           // Shell interface definitions
#include "AEEFile.h"			// File interface definitions
#include "AEENet.h"				// Socket interface definitions

#include "AEEStdLib.h"
#include "AEEMenu.h"
#include "AEEDisp.h"

#include "NetworkApp.bid"

#define MENU_EVT_CONNECT	401
#define MENU_EVT_GETHOST	402

typedef enum
{
   NET_NONE,
   NET_GETHOST,
   NET_CONNECT
} NetProcess;

typedef struct _CINetworkSample
{
	AEEApplet		a;          // Mandatory first AEEApplet data member
	IMenuCtl*		m_pIMenu;
	ISocket *		m_pISocket;
	INetMgr *		m_pINetMgr;
		
	AEEDNSResult	dnsresult;
	AEECallback		cbkLookup;
	INAddr			addrsLookup[8];

	int             m_nLineHeight;
	AEEDeviceInfo   m_dInfo;
	INPort			wPort;
	INAddr			dwAddr;
	NetProcess		status;
//   uint16      APIFnType;    // Storage for API function type selected
                             // by user.
} CINetworkSample;
 

#define USAGE_HOST      "211.41.45.249"   // "satriani.qualcomm.com"
#define USAGE_HOSTURL	"www.gaeasoft.co.kr"
#define USAGE_PORT      7                 // Server echo port

// A few utility macros; 
#define ISDIGIT(c)  ( (unsigned) ((c) - '0') < 10)
#define ISALPHA(c)  ( (unsigned) ( ((c)|32) - 'a') < 26 )
#define ISALNUM(c)  ( ISDIGIT(c) || ISALPHA(c) )
#define min(a,b)  ((a) < (b) ? (a) : (b))
#define ARRAYSIZE(a)   (sizeof(a) / sizeof((a)[0]))

/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
static boolean NetworkSample_HandleEvent(IApplet * pi, AEEEvent eCode, 
                                      uint16 wParam, uint32 dwParam);

static void DisplayMenu(CINetworkSample *pMe);
static boolean NetworkSample_InitAppData(IApplet* pi);
static void NetworkSample_FreeAppData(IApplet* pi);
static void DisplayOutput(IApplet * pMe, int nline, char *pszStr, RGBVAL clrColor);
void CommonConnectCB(void * pUser, int iError);
void CommonReadCBFn (void *pUser);
void CommonWriteCBFn(void *pUser);
void DisconnectSocket(CINetworkSample * pMe);

INAddr ConvertToINAddr(char *psz);
void ConvertToIPAddr(INAddr inaddr, char *psz);
			  
void GetHostByNameCBFn(void * pUser);

/*===============================================================================
FUNCTION DEFINITIONS
=============================================================================== */

/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to IDisplay_InitAppData(). 
   This is done as follows: InitAppData() is called to initialize AppletData 
   instance. It is app developers responsibility to fill-in app data initialization 
   code of InitAppData(). App developer is also responsible to release memory 
   allocated for data contained in AppletData -- this can be done in 
   IDisplay_FreeAppData().

PROTOTYPE:
   int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell object. 

	pIModule: pin]: Contains pointer to the IModule object to the current module to which
	this app belongs

	ppObj: [out]: On return, *ppObj must point to a valid IApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  AEE_SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was
     successful
  EFAILED: If the app does not need to be loaded or if errors occurred in 
     AEEApplet_New(). If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   if(ClsId == AEECLSID_NETWORKAPP){
      if(AEEApplet_New(sizeof(CINetworkSample), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)NetworkSample_HandleEvent,(PFNFREEAPPDATA)NetworkSample_FreeAppData)
         == TRUE)
      {
		 // Add your code here .....
		 if (NetworkSample_InitAppData((IApplet*)*ppObj) == TRUE)
			return (AEE_SUCCESS);
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION NetworkSample_HandleEvent

DESCRIPTION
	This is the EventHandler for this app. All events to this app are handled in this
	function. All APPs must supply an Event Handler.

PROTOTYPE:
	boolean NetworkSample_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)

PARAMETERS:
	pi: Pointer to the AEEApplet structure. This structure contains information specific
	to this applet. It was initialized during the AEEClsCreateInstance() function.

	ecode: Specifies the Event sent to this applet

   wParam, dwParam: Event specific data.

DEPENDENCIES
  none

RETURN VALUE
  TRUE: If the app has processed the event
  FALSE: If the app did not process the event

SIDE EFFECTS
  none
===========================================================================*/
static boolean NetworkSample_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
	CINetworkSample * pMe = (CINetworkSample *)pi;
   
	switch (eCode) 
	{
      case EVT_APP_START:                        
		  { 
		    // Add your code here .....
			
			/*INetMgr *pINetMgr = NULL;

			const char hostName [] = "www.gaeasoft.com";

			ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);
						
			if (pINetMgr)
			{
				// Store a pointer to the NetMgr in the AEEApplet struct. This 
				// will be used in the GetHostByNameCBFn callback fucntions.
				//Initialize the CB structure
				pMe->m_pINetMgr = pINetMgr;
				CALLBACK_Init(&pMe->cbkLookup, GetHostByNameCBFn, pMe);
				INETMGR_GetHostByName(pINetMgr,&pMe->dnsresult, hostName, &pMe->cbkLookup);
				INETMGR_Release(pMe->m_pINetMgr);
			}*/
			DisplayMenu(pMe);
      		return(TRUE);
		  }
      case EVT_KEY:
			if (pMe->m_pIMenu)
			{
				if ((IMENUCTL_IsActive(pMe->m_pIMenu) == FALSE) && (wParam == AVK_RIGHT))
				{
					DisconnectSocket(pMe);		
					IMENUCTL_SetActive(pMe->m_pIMenu, TRUE);
					IMENUCTL_Redraw(pMe->m_pIMenu);
				}
			    return IMENUCTL_HandleEvent(pMe->m_pIMenu, EVT_KEY, wParam, 0);
			}
			else 
				return FALSE;
	  case EVT_APP_STOP:

		    // Add your code here .....
			
         return TRUE;
	  case EVT_COMMAND:
		  {
			  switch(wParam)
			  {
				  case MENU_EVT_CONNECT:
					{
						INetMgr * pINetMgr = NULL;
						ISocket * pISocket = NULL;
						int iRetVal;
						
						INAddr nodeINAddr;
						uint16 nPort;
						
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						DisplayOutput((IApplet *)pMe, 1, "INetMgr, ISocket ̿ϱ", MAKE_RGB(0,0,0));
						
						ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);
						pISocket = INETMGR_OpenSocket (pINetMgr, AEE_SOCK_STREAM); 
						
						pMe->m_pINetMgr = pINetMgr;
						pMe->m_pISocket = pISocket;
						
						nodeINAddr = ConvertToINAddr(USAGE_HOST);
						nPort = HTONS(USAGE_PORT);
		
						if((iRetVal = ISOCKET_Connect (pISocket, nodeINAddr, nPort, CommonConnectCB, (void *)pMe)) != AEE_NET_SUCCESS)
						{
							DisplayOutput((IApplet *)pMe, 7, "Network Error", MAKE_RGB(0xff, 0, 0));
						}
						pMe->status = NET_CONNECT;
						return TRUE;
					}
				  case MENU_EVT_GETHOST:
					{
						INetMgr *pINetMgr = NULL;
						const char hostName [] = "littlekamja.gaeasoft.co.kr";
						
						IDISPLAY_ClearScreen (pMe->a.m_pIDisplay);
						IMENUCTL_SetProperties(pMe->m_pIMenu, MP_NO_REDRAW);
						IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
						DisplayOutput((IApplet *)pMe, 1, "GetHostByName ̿ϱ", MAKE_RGB(0,0,0));
						
						ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_NET, (void **)&pINetMgr);

			            if (pINetMgr)	
						{
							pMe->m_pINetMgr = pINetMgr;
							pMe->status = NET_GETHOST;
							CALLBACK_Init(&pMe->cbkLookup, GetHostByNameCBFn, pMe);
							INETMGR_GetHostByName(pINetMgr,&pMe->dnsresult, hostName, &pMe->cbkLookup);
						}
						return TRUE;
					}
				  default:
					  return TRUE;
			  }
		  }			
      default:
         break;
   }
   return FALSE;
}


static void DisplayMenu(CINetworkSample *pMe)
{
	AEERect rc;
	AECHAR  szBuf[50];
	AEEMenuColors mclr;
	
	if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu) != SUCCESS)
		return;
	
	// Set display rectangle
	SETAEERECT (&rc, 0, 0, pMe->m_dInfo.cxScreen, pMe->m_dInfo.cyScreen);

	// õ ޴  Ѵ. 
	mclr.wMask = (MC_BACK | MC_TEXT | MC_SEL_BACK | MC_SEL_TEXT | MC_FRAME);			
	mclr.cBack = RGB_WHITE;
	mclr.cText = RGB_BLACK;
	mclr.cSelBack = MAKE_RGB (146, 109, 255);
	mclr.cSelText = RGB_BLACK;
	mclr.cFrame = RGB_WHITE;

	IMENUCTL_SetColors (pMe->m_pIMenu, &mclr);
		
	// ޴ Ʈ ŸƲ Ѵ
	STREXPAND((byte *)"INegMgr, ISocket", STRLEN("INegMgr, ISocket"), szBuf, sizeof(szBuf));
	IMENUCTL_SetTitle(pMe->m_pIMenu, NULL, 0, szBuf);
	IMENUCTL_SetRect(pMe->m_pIMenu, &rc);  
	
	STREXPAND((byte *)"1.  ϱ", STRLEN("1.  ϱ"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_CONNECT, szBuf, 0);
	
	STREXPAND((byte *)"2. GetHostByName ̿ϱ", STRLEN("2. GetHostByName ̿ϱ"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_GETHOST, szBuf, 0);
	/*STREXPAND((byte *)"2.    ϱ", STRLEN("2.    ϱ"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_PLAYTONELIST, szBuf, 0);

	STREXPAND((byte *)"3.  ̽  ", STRLEN("3.  ̽  "), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_GETDEVICE, szBuf, 0);
	
	STREXPAND((byte *)"4.  ߻Ű", STRLEN("4.  ߻Ű"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_VIBRATE, szBuf, 0);
	
	STREXPAND((char *)"5.  ϱ", STRLEN("5.  ϱ"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_VOLUME, szBuf, 0);
	
	STREXPAND((char *)"6. QCELP   ϱ", STRLEN("6. QCELP   ϱ"), szBuf, sizeof(szBuf));
	IMENUCTL_AddItem(pMe->m_pIMenu, 0, 0, MENU_EVT_SOUNDPLAYER, szBuf, 0);
	*/
	IMENUCTL_SetActive(pMe->m_pIMenu,TRUE);
}

static boolean NetworkSample_InitAppData(IApplet* pi)
{
	CINetworkSample* pMe = (CINetworkSample*)pi;
	int pnAscent = 0;
	int pnDescent = 0;
	// Initialize the MenuCtl pointer to NULL
	pMe->m_pIMenu = NULL;
	pMe->m_pINetMgr = NULL;
	pMe->m_pISocket = NULL;
	pMe->status = NET_NONE;
	pMe->m_nLineHeight = IDISPLAY_GetFontMetrics (pMe->a.m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);
	ISHELL_GetDeviceInfo(pMe->a.m_pIShell,&pMe->m_dInfo);
	return TRUE;

}
static void NetworkSample_FreeAppData(IApplet* pi)
{
	CINetworkSample* pMe = (CINetworkSample*)pi;
	// ޴ Ʈ   ʾҴٸ Ѵ.
	if (pMe->m_pIMenu != NULL)
	{
		// ޴ Ʈ ϴ Լ
		IMENUCTL_Release (pMe->m_pIMenu);
		pMe->m_pIMenu = NULL;
	}
	
    if(pMe->status == NET_GETHOST)
		CALLBACK_Cancel(&pMe->cbkLookup);

	if (pMe->m_pISocket != NULL)
    {
       ISOCKET_Release (pMe->m_pISocket);
       pMe->m_pISocket = NULL;
    }
   
    if (pMe->m_pINetMgr != NULL)
    {
       INETMGR_SetLinger(pMe->m_pINetMgr,0);
	   INETMGR_Release (pMe->m_pINetMgr);
       pMe->m_pINetMgr = NULL;
    }
}

static void DisplayOutput(IApplet * pi, int nline, char *pszStr, RGBVAL clrColor)
{
   AEEDeviceInfo di;        // Device Info
   AECHAR szBuf[200] = {0}; // a buffer that supports 200 char string
   AECHAR * psz = NULL;
   int pixelWidth;
   AEEFont font = AEE_FONT_NORMAL;
   int pnFits = 0, dy;
   int totalCh = 0;
   int charHeight = 0;      // Stores the char height in pixels for given font
   int pnAscent = 0;        // Stores the ascent in number of pixels
   int pnDescent = 0;       // Stores the descent in number of pixels
	AEERect rc;

   AEEApplet * pMe = (AEEApplet*)pi;

   if (pMe == NULL)
      return;
	   
   // Get device information
   ISHELL_GetDeviceInfo(pMe->m_pIShell,&di);

   // Get the font metrics info
   charHeight = IDISPLAY_GetFontMetrics (pMe->m_pIDisplay, AEE_FONT_NORMAL,
      &pnAscent, &pnDescent);

   // Convert to wide string (unicode)
   // STR_TO_WSTR ((char *)pszStr, szBuf, sizeof (szBuf));
   
   STREXPAND((byte *)pszStr, STRLEN(pszStr), szBuf, sizeof(szBuf));
   
   //STREXPAND("ISHELL ", sizeof("ISHELL "), szBuf, sizeof(szBuf));
   // If nlines is less than zero then print this string starting around the middle of 
   // the screen. Or else multiply nlines by 10 to decide the y coordinate of
   // the start of the string.
   if (nline < 0) {
      dy = di.cyScreen*2/5;
   }
   else{
      dy = nline * charHeight;
   }

   // psz keeps track of the point from which to write from the string buffer
   // in case the string does not fit one line and needs to wrap around in the
   // next line.
   psz = szBuf;
      
   // Need to calculate the lotal string length to decide if any wrapping
   // around is needed.
   totalCh = STRLEN ((char *)pszStr);

   // Keep displaying text string on multiple lines if the string can't be displayed
   // on one single line. Lines are spaced 15 pixels apart.
   while ((totalCh > 0) && (*psz != NULL))
   { 
      // Get information on how many characters will fit in a line.
      // Give the pointer to the buffer to be displayed, and the number of
      // pixels along the x axis you want to display the string in (max number)
      // pnFits will have the max number of chars that will fit in the maxWidth
      // number of pixels (given string can't fit in one line), or the number of 
      // chars in the string (if it does fit in one line). pnWidth gives the
      // number of pixels that will be used to display pnFits number of chars.
      pixelWidth = IDISPLAY_MeasureTextEx(pMe->m_pIDisplay,
                      font, 
                      (AECHAR *) psz,  // Start of the buffer to display,
                      -1,
                      di.cxScreen - 5, // maxWidth
                      &pnFits);         // Number of chars that will fit a line

      // If pnFits is zero there is something wrong in the input to above function. 
      // Normally this scenario should not occur. But, have the check anyway.
      if (pnFits == 0)
         return;

		SETAEERECT(&rc, 0, dy, di.cxScreen, charHeight);
		IDISPLAY_EraseRect(pMe->m_pIDisplay, &rc);
		IDISPLAY_SetColor(pMe->m_pIDisplay,CLR_USER_TEXT,clrColor);
        IDISPLAY_DrawText(pMe->m_pIDisplay, AEE_FONT_NORMAL, psz, pnFits, 5 /*start dx*/, 
         dy, 0 /* use default rectangle coordinates */, 0);

      psz += pnFits;      // move pointer to the next segment to be displayed
      totalCh -= pnFits;  // reduce the total number of characters to still display
      dy += charHeight;   // Place next line charHeight pixels below the 
                          // previous line. 
      IDISPLAY_Update(pMe->m_pIDisplay);
      if (totalCh < pnFits)
         pnFits = totalCh;  // if total number is less than pnFits, adjust pnFits
   }
   return;   
}

INAddr ConvertToINAddr(char *psz)
{
   INAddr ul = 0;
   int nByte = 0;
   char c;

   if(!psz)
      return 0;

   while (ISDIGIT(*psz)) {
      int n = 0;
      while ( ISDIGIT(c=*psz)) {
         n = n*10 + (c - '0');
         ++psz;
      }
      ((char*)&ul)[nByte++] = n;
 
      if (nByte == 4 || *psz != '.')
         break;
 
      ++psz;
   }
 
   if (nByte < 4 || ISALNUM(*psz))
      ul = 0xFFFFFFFF;    // Invalid address

   return ul;
}

void CommonConnectCB(void * pUser, int iError)
{
	CINetworkSample * pMe = (CINetworkSample*)pUser;
	int32 iRetValue;
	char pszData[20] = "Hello World";
	char szBuf[20] = {0};

	char *psz = NULL;
	int  cbWrite;
   
	psz = pszData;
	cbWrite = STRLEN (pszData);

	DisplayOutput((IApplet *)pMe, 3, "Ͽ  Ÿ", MAKE_RGB(0xff, 0, 0));
	
	iRetValue = ISOCKET_Write(pMe->m_pISocket, (byte *)psz, (uint16)cbWrite);
	if(iRetValue == AEE_NET_WOULDBLOCK)
	{
		ISOCKET_Writeable(pMe->m_pISocket, CommonWriteCBFn, (void *)pMe);
	}
	else if(iRetValue == AEE_NET_ERROR)
	{
		DisplayOutput((IApplet *)pMe, 4, "AEE_NET_ERROR", MAKE_RGB(0xff, 0, 0));
		DisconnectSocket(pMe);
		return;
	}
	else if(iRetValue > 0)
	{
		DisplayOutput((IApplet *)pMe, 4, (byte *)psz, MAKE_RGB(0xff, 0, 0));
	}

	DisplayOutput((IApplet *)pMe, 6, "Ͽ  Ÿ", MAKE_RGB(0xff, 0, 0));
	
	iRetValue = ISOCKET_Read(pMe->m_pISocket, (byte *)szBuf, sizeof(szBuf)) ;
	if(iRetValue == AEE_NET_WOULDBLOCK)
	{
		DisplayOutput((IApplet *)pMe, 7, "AEE_NET_WOULDBLOCK", MAKE_RGB(0xff, 0, 0));
		ISOCKET_Readable(pMe->m_pISocket, CommonReadCBFn, (void*)pMe);
	}
	else if(iRetValue == AEE_NET_ERROR)
	{
		DisplayOutput((IApplet *)pMe, 7, "AEE_NET_ERROR", MAKE_RGB(0xff, 0, 0));
		DisconnectSocket(pMe);
	}
	else if(iRetValue > 0)
	{
		DisplayOutput((IApplet *)pMe, 7, (byte *)szBuf, MAKE_RGB(0xff, 0, 0));
		DisconnectSocket(pMe);
	}
	else if(iRetValue == 0)
	{
		DisplayOutput((IApplet *)pMe, 7, "   ", MAKE_RGB(0xff, 0, 0));
		DisconnectSocket(pMe);
	}

	/*   ڵ DisconnectSocket Լ üѴ */
	/*
	DisplayOutput((IApplet *)pMe, 8, (byte *)szBuf, MAKE_RGB(0xff, 0, 0));

	if (pMe->m_pISocket != NULL)
	{
		ISOCKET_Release (pMe->m_pISocket);
		pMe->m_pISocket = NULL;
	}
   
	if (pMe->m_pINetMgr != NULL)
	{
		INETMGR_Release (pMe->m_pINetMgr);
		pMe->m_pINetMgr = NULL;
	} 
	*/ 
}

void GetHostByNameCBFn(void * pUser)
{
	CINetworkSample * pMe = (CINetworkSample*)pUser;
	ISocket *pISocket = NULL;
	int32 iRetVal = 0;
	char *szMsg = MALLOC(100);

	int i;
	if(!szMsg)
		return;

	switch(pMe->dnsresult.nResult)
	{
		case AEE_NET_BADDOMAIN:
		{
			SPRINTF(szMsg,"Host name is mal-formed; not a valid host name");
			DisplayOutput((IApplet*)pMe,4,szMsg, MAKE_RGB(0xff, 0, 0));
			//FREE(szMsg);
			//return;
			break;
		}
		case AEE_NET_UNKDOMAIN: 
		{
			SPRINTF(szMsg,"Unknown host, or one without IP addresses");
			DisplayOutput((IApplet*)pMe,4,szMsg, MAKE_RGB(0xff, 0, 0));
			//FREE(szMsg);
			//return;
			break;
		}
		case AEE_NET_ETIMEDOUT:
		{	
			SPRINTF(szMsg,"No response was seen within the maximum time limit");
			DisplayOutput((IApplet*)pMe,4,szMsg, MAKE_RGB(0xff, 0, 0));
			//FREE(szMsg);
			//return;
			break;
		}
		case EUNSUPPORTED:
		{
			SPRINTF(szMsg,"No DNS servers are configured");
			DisplayOutput((IApplet*)pMe,4,szMsg, MAKE_RGB(0xff, 0, 0));
			//FREE(szMsg);
			//return;
			break;
		}
		case ENOMEMORY: 
		{
			SPRINTF(szMsg,"Could not perform query due to allocation failure");
			DisplayOutput((IApplet*)pMe,4,szMsg, MAKE_RGB(0xff, 0, 0));
			//FREE(szMsg);
			//return;
			break;
		}
		default:
		{
			if(pMe->dnsresult.nResult > 0)
			{
				ConvertToIPAddr(pMe->dnsresult.addrs[0], szMsg);
				DisplayOutput((IApplet*)pMe,10,szMsg, MAKE_RGB(0xff, 0, 0xff));
				
				pMe->dwAddr = pMe->dnsresult.addrs[0];

				pISocket = INETMGR_OpenSocket (pMe->m_pINetMgr, AEE_SOCK_STREAM); 
						
				pMe->m_pISocket = pISocket;
						
				//nodeINAddr = ConvertToINAddr(USAGE_HOST);
				//nPort = HTONS(USAGE_PORT);
	
				if((iRetVal = ISOCKET_Connect (pISocket, pMe->dwAddr, HTONS(7), CommonConnectCB, (void *)pMe)) != AEE_NET_SUCCESS)
				{
					DisplayOutput((IApplet *)pMe, 7, "Network Error", MAKE_RGB(0xff, 0, 0));
				}
			}
			else
			{
				SPRINTF(szMsg,"Unknown Error");
				DisplayOutput((IApplet*)pMe,4,szMsg, MAKE_RGB(0xff, 0, 0));
			//	FREE(szMsg);
			//	return;
			}
			break;
		}
	}

	FREE(szMsg);

	return;
}

void ConvertToIPAddr(INAddr inaddr, char *psz)
{
   int i;
   int x[4] = {0};
   char ch[30];
   char ch1[50];

   for (i = 0; i < 4; i++)
   {
      x[i] = (int)(inaddr & 0xff);
      inaddr = inaddr >> 8;     
   }


   for (i = 3; i >= 0; i--)
   {
      SPRINTF (ch,"%d",x[i]);
      if (i != 0)
        STRCAT (ch, ".");

      if (i == 3)
         STRCPY (ch1, ch);
      else
         STRCAT(ch1, ch);
   }

   STRCPY (psz, ch1);   
}

void CommonWriteCBFn(void *pUser)
{
	CINetworkSample * pMe = (CINetworkSample*)pUser;
	int32 iRetValue;
	char pszData[20] = "Hello World";
	char *psz = NULL;
	int  cbWrite;
   	ISocket *pISocket = pMe->m_pISocket;

	psz = pszData;
	cbWrite = STRLEN (pszData);
	
	iRetValue = ISOCKET_Write(pISocket, (byte *)psz, (uint16)cbWrite);

	if(iRetValue == AEE_NET_WOULDBLOCK)
	{
		ISOCKET_Writeable(pMe->m_pISocket, CommonWriteCBFn, (void *)pMe);
	}
	else if(iRetValue == AEE_NET_ERROR)
	{
		DisplayOutput((IApplet *)pMe, 4, "AEE_NET_ERROR", MAKE_RGB(0xff, 0, 0));
		DisconnectSocket(pMe);
	}
	else if(iRetValue > 0)
	{
		DisplayOutput((IApplet *)pMe, 4, (byte *)psz, MAKE_RGB(0xff, 0, 0));
	}
}


void CommonReadCBFn (void *pUser)
{
	CINetworkSample * pMe = (CINetworkSample*)pUser;
	int32 iRetValue;
	char szBuf[100] = {0};
	
	ISocket *piSock = pMe->m_pISocket;
    
    iRetValue = ISOCKET_Read(piSock, (byte *)szBuf, sizeof(szBuf));

    if (iRetValue == AEE_NET_WOULDBLOCK)
    {
		ISOCKET_Readable(piSock, CommonReadCBFn, (void*)pMe); 
		return;
	}
    else if (iRetValue == AEE_NET_ERROR) 
    {
        DisplayOutput ((IApplet *)pMe, 9, "Read Error!",MAKE_RGB(0, 0, 0xff));
        DisconnectSocket (pMe);
    }
    else if (iRetValue > 0) // data has been read
    {
	   	// The data has been read from the socket. Display Data read.
        DisplayOutput ((IApplet *)pMe, 9, (byte *)szBuf, MAKE_RGB(0, 0, 0xff));
        DisconnectSocket (pMe);
        // If we wanted to read more data, we could loop here or call Readable() 
        // here and return.
    }
    else  // rv == 0
    {
        // There is no more data to be received. 
        // The peer has shut down the connection.
        DisplayOutput ((IApplet *)pMe, 9, " Ͱ .", MAKE_RGB(0, 0, 0xff));
		DisconnectSocket (pMe);
    }
}

void DisconnectSocket(CINetworkSample * pMe)
{
   	if(pMe->status == NET_GETHOST)
		CALLBACK_Cancel(&pMe->cbkLookup);

	if (pMe->m_pISocket != NULL)
	{
		ISOCKET_Release (pMe->m_pISocket);
		pMe->m_pISocket = NULL;
	}
	if (pMe->m_pINetMgr != NULL)
	{
		INETMGR_SetLinger(pMe->m_pINetMgr,0);
		INETMGR_Release (pMe->m_pINetMgr);
		pMe->m_pINetMgr = NULL;
	}
}