/*===============================================================================
INCLUDES AND VARIABLE DEFINITIONS
=============================================================================== */
#include "AEEAppGen.h"          // Applet interface definitions
#include "AEEStdLib.h"
#include "dialogcontrol.bid"		// applet class ID
#include "dialogcontrol_res.h"
#include "AEEMenu.h"		// Menu Services

typedef struct _CIDialogCtlApp {
   AEEApplet	      a;
   IMenuCtl *  m_pIMenu;   // Pointer to main app menu
   //IMenuCtl *  m_pStdMenu; // Pointer to standard menu used in examples
   //IMenuCtl *  m_pSKMenu;  // Pointer to softkey menu used in examples
   //IMenuCtl *  m_pListMenu; // Pointer to list-control menu used in examples
   //IMenuCtl *  m_pIconMenu; // Pointer to icon-view menu used in examples
   //AEERect     m_rc;
} CDialogCtlApp;

/*-------------------------------------------------------------------
Function Prototypes
-------------------------------------------------------------------*/
static boolean DialogControl_HandleEvent(IApplet * pi, AEEEvent eCode, 
                                      uint16 wParam, uint32 dwParam);

/*===============================================================================
FUNCTION DEFINITIONS
=============================================================================== */
/*===========================================================================

FUNCTION: AEEClsCreateInstance

DESCRIPTION
	This function is invoked while the app is being loaded. All Modules must provide this 
	function. Ensure to retain the same name and parameters for this function.
	In here, the module must verify the ClassID and then invoke the AEEApplet_New() function
	that has been provided in AEEAppGen.c. 

   After invoking AEEApplet_New(), this function can do app specific initialization. In this
   example, a generic structure is provided so that app developers need not change app specific
   initialization section every time except for a call to InitAppData(). This is done as follows:
   InitAppData() is called to initialize AppletData instance. It is app developers 
   responsibility to fill-in app data initialization code of InitAppData(). App developer
   is also responsible to release memory allocated for data contained in AppletData -- this can be
   done in FreeAppData().

PROTOTYPE:
	int AEEAppCreateInstance(AEECLSID clsID, IShell* pIShell, IModule* pIModule, 
										  IApplet** ppApplet)

PARAMETERS:
	clsID: [in]: Specifies the ClassID of the applet which is being loaded

	pIShell: [in]: Contains pointer to the IShell interface. 

	pIModule: pin]: Contains pointer to the IModule interface to the current module to which
	this app belongs

	ppApplet: [out]: On return, *ppApplet must point to a valid AEEApplet structure. Allocation
	of memory for this structure and initializing the base data members is done by AEEApplet_New().

DEPENDENCIES
  none

RETURN VALUE
  SUCCESS: If the app needs to be loaded and if AEEApplet_New() invocation was successful
  EFAILED: If the app does not need to be loaded or if errors occurred in AEEApplet_New().
  If this function returns FALSE, the app will not be loaded.

SIDE EFFECTS
  none
===========================================================================*/
int AEEClsCreateInstance(AEECLSID ClsId,IShell * pIShell,IModule * po,void ** ppObj)
{
   *ppObj = NULL;
		
   if(ClsId == AEECLSID_DIALOGCONTROL){
      if(AEEApplet_New(sizeof(CDialogCtlApp), ClsId, pIShell,po,(IApplet**)ppObj,
         (AEEHANDLER)DialogControl_HandleEvent,NULL)
         == TRUE)
      {
         return (AEE_SUCCESS);
      }
   }
	return (EFAILED);
}

/*===========================================================================

FUNCTION DialogControl_HandleEvent
  
DESCRIPTION
   This is the EventHandler for this app. All events to this app are handled in this
   function. All APPs must supply an Event Handler.
    
PROTOTYPE:
   boolean HelloWorld_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
      
PARAMETERS:
   pi: Pointer to the AEEApplet structure. This structure contains information specific
   to this applet. It was initialized during the AppCreateInstance() function.
        
   ecode: Specifies the Event sent to this applet
          
   wParam, dwParam: Event specific data.
            
DEPENDENCIES
   none
              
RETURN VALUE
   TRUE: If the app has processed the event
   FALSE: If the app did not process the event
                
SIDE EFFECTS
   none
===========================================================================*/
static boolean DialogControl_HandleEvent(IApplet * pi, AEEEvent eCode, uint16 wParam, uint32 dwParam)
{  
   CDialogCtlApp * pMe = (CDialogCtlApp*)pi;
      
   AECHAR szBuf[] = {'H','e','l','l','o',' ','W','o', 'r', 'l', 'd', '\0'};

   switch (eCode) 
	{
      case EVT_APP_START:                        
			// Create the first dialog
			if(ISHELL_CreateInstance(pMe->a.m_pIShell, AEECLSID_MENUCTL, (void **)&pMe->m_pIMenu) != SUCCESS)
				return TRUE;
   			//IMENUCTL_SetActive(pMe->m_pIMenu, FALSE);
			IDISPLAY_DrawText(pMe->a.m_pIDisplay, AEE_FONT_BOLD, szBuf, -1, 0, 0, 0, IDF_ALIGN_CENTER | IDF_ALIGN_MIDDLE);
            IDISPLAY_Update (pMe->a.m_pIDisplay);
			//ISHELL_CreateDialog(pMe->a.m_pIShell, DIALOGCONTROL_RES_FILE, IDS_TEST_DIALOG, NULL);
			return(TRUE);
      case EVT_APP_STOP:
         return TRUE;
      default:
         break;
   }

   return FALSE;
}


